/**
 ******************************************************************************
 * @file    testVideo.c
 * @version V1.0.0
 * @date    2017/11/10
 * @brief   This file provides an payload example of sending video flow data to mobile APP via
 *           network port video channel of PSDK.
 *
 * @copyright (c) 2017-2018 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define READ_SIZE 20480000
#define MAX_SEND_PACK 8192

//port and ip
#define PORT 23003
#define SERVER_IP "192.168.5.10"
#define FILE_NAME "test.h264"

//H.264 format related
#define FRAME_HEAD_LEN  4

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
char SPS_Head[FRAME_HEAD_LEN] = {0x00, 0x00, 0x01, 0x67};
char PPS_Head[FRAME_HEAD_LEN] = {0x00, 0x00, 0x01, 0x68};
char I_Head[FRAME_HEAD_LEN] = {0x00, 0x00, 0x01, 0x65};
char P_Head[FRAME_HEAD_LEN] = {0x00, 0x00, 0x01, 0x61};
char P_Head2[FRAME_HEAD_LEN] = {0x00, 0x00, 0x01, 0x41};

char buffer[READ_SIZE];

/* Exported variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
/* Exported functions --------------------------------------------------------*/

/**
 * @brief Main function.
 * @return
 */
int main(int argc, char **argv)
{
    FILE *pF;
    int i, j;

    bzero(buffer, sizeof(buffer));

    pF = fopen(FILE_NAME, "rb+");
    if (pF == NULL) {
        printf("No File!\r\n");
    }

    int s, len;
    struct sockaddr_in addr;
    int addr_len = sizeof(struct sockaddr_in);

    if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("socket");
        exit(1);
    }

    bzero(&addr, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_port = htons(PORT);
    addr.sin_addr.s_addr = inet_addr(SERVER_IP);

    len = fread(buffer, 1, READ_SIZE, pF);

    if (len < 0) {
        printf("read length error!\r\n");
    } else {
        printf("file len = %d B\r\n", len);
    }

    i = 0;
    j = 0;

    while (1) {
        while (memcmp(buffer + i, SPS_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + i, PPS_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + i, I_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + i, P_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + i, P_Head2, FRAME_HEAD_LEN) != 0) {
            i++;

            if (i > len) {
                printf("Send finish\r\n");
                fclose(pF);

                printf("Resend\r\n");
                pF = fopen(FILE_NAME, "rb+");

                i = 0;
                j = 0;

                continue;
            }
        }

        j = i + FRAME_HEAD_LEN;
        while (memcmp(buffer + j, SPS_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + j, PPS_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + j, I_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + j, P_Head, FRAME_HEAD_LEN) != 0
               && memcmp(buffer + j, P_Head2, FRAME_HEAD_LEN) != 0) {
            j++;

            if (j > len) {
                printf("send finish\r\n");
                fclose(pF);

                printf("Resend\r\n");
                pF = fopen(FILE_NAME, "rb+");

                i = 0;
                j = 0;

                continue;
            }
        }

        if (memcmp(buffer + i, SPS_Head, FRAME_HEAD_LEN) == 0) {
            printf("send SPS frame, len = %d\r\n", j - i);

            while (j - i > MAX_SEND_PACK) {
                sendto(s, buffer + i, MAX_SEND_PACK, 0, (struct sockaddr *) &addr, addr_len);
                i += MAX_SEND_PACK;
            }

            sendto(s, buffer + i, j - i, 0, (struct sockaddr *) &addr, addr_len);
            usleep(33000);
        } else if (memcmp(buffer + i, PPS_Head, FRAME_HEAD_LEN) == 0) {
            printf("send PPS frame, len = %d\r\n", j - i);

            while (j - i > MAX_SEND_PACK) {
                sendto(s, buffer + i, MAX_SEND_PACK, 0, (struct sockaddr *) &addr, addr_len);
                i += MAX_SEND_PACK;
            }

            sendto(s, buffer + i, j - i, 0, (struct sockaddr *) &addr, addr_len);
            usleep(33000);
        } else if (memcmp(buffer + i, I_Head, FRAME_HEAD_LEN) == 0) {
            printf("send I frame, len = %d\r\n", j - i);

            while (j - i > MAX_SEND_PACK) {
                sendto(s, buffer + i, MAX_SEND_PACK, 0, (struct sockaddr *) &addr, addr_len);
                i += MAX_SEND_PACK;
            }

            sendto(s, buffer + i, j - i, 0, (struct sockaddr *) &addr, addr_len);
            usleep(33000);
        } else if (memcmp(buffer + i, P_Head, FRAME_HEAD_LEN) == 0
                   || memcmp(buffer + i, P_Head2, FRAME_HEAD_LEN) == 0) {
            printf("send P frame, len = %d\r\n", j - i);

            while (j - i > MAX_SEND_PACK) {
                sendto(s, buffer + i, MAX_SEND_PACK, 0, (struct sockaddr *) &addr, addr_len);
                i += MAX_SEND_PACK;
            }

            sendto(s, buffer + i, j - i, 0, (struct sockaddr *) &addr, addr_len);
            usleep(33000);
        }

        i = j;
    }
}
